@@ -26,7 +26,12 @@ then
 fi
 if [ -z $SIESTA_PS_PATH ]
 then
-  export SIESTA_PS_PATH=$(get_abs_filename "../../Pseudos")
+  if [ -z $PS_PATH_HINT ]
+  then
+    export SIESTA_PS_PATH=$(get_abs_filename "../../Pseudos")
+  else
+    export SIESTA_PS_PATH=$(get_abs_filename ${PS_PATH_HINT})
+  fi
 fi
 
 # Failsafe for weird OMP behaviour.
